module _9.mainform;

import std.string;
import std.conv;
import std.regexp;
import dfl.all;
import _9.res;

class MainForm : Form {
	const char[] winMessage = "Cirno WIN!!";
	const char[] loseMessage = "Cirno LOSE...";

	this(){
		text = "Cal⑨lator -「チルノが敵を倒すスレ」のダメージ計算を⑨なりに頑張ります-";
		size = Size(640, 480);
		
		with(enemyNameLabel = new Label()){
			text = "敵の名前：";
			autoSize = true;
			parent = this;
		}
		with(enemyNameBox = new TextBox()){
			text = "コンバット越前";
			parent = this;
		}
		with(enemyHPLabel = new Label()){
			text = "敵のHP：";
			autoSize = true;
			parent = this;
		}
		with(enemyHPBox = new TextBox()){
			text = "10000";
			parent = this;
		}
		
		with(cirnoNameLabel = new Label()){
			text = "⑨：";
			autoSize = true;
			parent = this;
		}
		with(cirnoNameBox = new TextBox()){
			cirnoNameBox.text = "チルノ";
			cirnoNameBox.readOnly = true;
			cirnoNameBox.enabled = false;
			parent = this;
		}
		with(cirnoHPLabel = new Label()){
			text = "チルノのHP：";
			autoSize = true;
			parent = this;
		}
		with(cirnoHPBox = new TextBox()){
			text = "99";
			parent = this;
		}
		
		with(inputBoxLabel = new Label()){
			text = "ここに計算したいレスの範囲をコピペして「計算」すると：";
			autoSize = true;
			parent = this;
		}
		with(inputBox = new RichTextBox()){
			multiline = true;
			size = Size(size.width, 100);
			parent = this;
		}

		with(outputBoxLabel = new Label()){
			text = "ここに結果のテンプレが出力されるのさ：";
			autoSize = true;
			parent = this;
		}
		with(outputBox = new RichTextBox()){
			multiline = true;
			parent = this;
		}
		
		with(calcButton = new Button()){
			text = "計算(&9)";
			click ~= &calc;
			parent = this;
		}
		
		onResize(null);
	}

	override void onResize(EventArgs arg){
		
		int enemyHeight = max([enemyNameLabel.height, enemyNameBox.height, enemyHPLabel.height, enemyHPBox.height]);
		int cirnoHeight = max([cirnoNameLabel.height, cirnoNameBox.height, cirnoHPLabel.height, cirnoHPBox.height]);
		int nameLabelWidth = max([enemyNameLabel.width, cirnoNameLabel.width]);
		int nameBoxWidth = max([enemyNameBox.width, cirnoNameBox.width]);
		int hpLabelWidth = max([enemyHPLabel.width, cirnoHPLabel.width]);
		int hpBoxWidth = max([enemyHPBox.width, cirnoHPBox.width]);

		int spaceHeight = clientSize.height - (
			enemyHeight +
			cirnoHeight +
			inputBoxLabel.height +
			inputBox.height +
			outputBoxLabel.height +
			calcButton.height);
		
		Point pos = Point(0, 0);

		enemyNameLabel.location = Point(pos.x + nameLabelWidth - enemyNameLabel.width, pos.y + (enemyHeight - enemyNameLabel.height) / 2);
		pos.x += nameLabelWidth;
		enemyNameBox.location = pos;
		pos.x += nameBoxWidth;
		enemyHPLabel.location = Point(pos.x + hpLabelWidth - enemyHPLabel.width, pos.y + (enemyHeight - enemyHPLabel.height) / 2);
		pos.x += hpLabelWidth;
		enemyHPBox.location = pos;
		pos.x = 0;
		pos.y += enemyHeight;

		cirnoNameLabel.location = Point(pos.x + nameLabelWidth - cirnoNameLabel.width, pos.y + (cirnoHeight - cirnoNameLabel.height) / 2);
		pos.x += nameLabelWidth;
		cirnoNameBox.location = pos;
		pos.x += nameBoxWidth;
		cirnoHPLabel.location = Point(pos.x + hpLabelWidth - cirnoHPLabel.width, pos.y + (cirnoHeight - cirnoHPLabel.height) / 2);
		pos.x += hpLabelWidth;
		cirnoHPBox.location = pos;
		pos.x = 0;
		pos.y += cirnoHeight;

		inputBoxLabel.location = pos;
		pos.y += inputBoxLabel.height;
		
		inputBox.bounds = Rect(0, pos.y, clientSize.width, inputBox.height);
		pos.y += inputBox.height;
		
		outputBoxLabel.location = pos;
		pos.y += outputBoxLabel.height;
		
		outputBox.bounds = Rect(0, pos.y, clientSize.width, spaceHeight);
		pos.y += outputBox.height;
		
		calcButton.location = pos;
		pos.y += calcButton.height;
	}
	
	static int max(int[] values){
		int result = 0;
		foreach(int value; values){
			if(result < value){
				result = value;
			}
		}
		return result;
	}
	unittest{
		assert(max([10, 30, 20]) == 30);
	}
	
	void calc(Object sender, EventArgs e){
		if(!validate()){
			return;
		}

		int enemyHP = enemyHPValue;
		int cirnoHP = cirnoHPValue;

		Res[] resList = Res.parse(inputBox.text);
		foreach(res; resList){
			outputBox.appendText(res.toText() ~ newline);
			res.result.apply(enemyHP, cirnoHP);
			if(enemyHP <= 0 || cirnoHP <= 0){
				break;
			}
		}
		
		printHP(enemyHP, cirnoHP);
		
		if(enemyHP <= 0){
			printWin();
		} else if(cirnoHP <= 0){
			printLose();
		}
	}
	
	int enemyHPValue(){
		return toInt(enemyHPBox.text);
	}
	int cirnoHPValue(){
		return toInt(cirnoHPBox.text);
	}
	
	void printHP(int enemyHP, int cirnoHP){
		outputBox.appendText(newline);
		outputBox.appendText(format("%s : %d-%d=%d", enemyNameBox.text, enemyHPValue, enemyHPValue - enemyHP ,enemyHP) ~ newline);
		outputBox.appendText(format("Cirno : %d%+d=%d", cirnoHPValue, cirnoHP - cirnoHPValue, cirnoHP) ~ newline);
		outputBox.appendText(newline);
	}
	
	void printWin(){
		outputBox.appendText(winMessage);
	}
	void printLose(){
		outputBox.appendText(loseMessage);
	}
	
	bool validate(){
		RegExp numberPattern = RegExp(r"\d+");
		
		if(!numberPattern.test(enemyHPBox.text)){
			return false;
		}
		if(!numberPattern.test(cirnoHPBox.text)){
			return false;
		}
		return true;
	}
	
private:
	Label enemyNameLabel;
	TextBox enemyNameBox;
	Label enemyHPLabel;
	TextBox enemyHPBox;

	Label cirnoNameLabel;
	TextBox cirnoNameBox;
	Label cirnoHPLabel;
	TextBox cirnoHPBox;

	Label inputBoxLabel;
	RichTextBox inputBox;
	Label outputBoxLabel;
	RichTextBox outputBox;
	Button calcButton;
}
