module _9.res;

import std.string;
import std.regexp;
import std.conv;
import _9.result;

class Res{
public:
	static Res[] parse(char[] text){
		RegExp pattern = RegExp(r"^(\d+) 名前：.+(\d\d:\d\d:\d\d) \[ (.{8}) \]$");
		
		Res[] results;
		foreach(line; splitlines(text)){
			if(!pattern.test(line)){
				continue;
			}
			
			int number = toInt(pattern.match(1));
			char[] time = pattern.match(2);
			char[] id = pattern.match(3);
			
			if(!duplicate(results, id)){
				results ~= new Res(number, time, id);
			}
		}
		return results;
	}
	
	static bool duplicate(Res[] results, char[] id){
		foreach(result; results){
			if(result.id == id){
				return true;
			}
		}
		return false;
	}
	
	override int opEquals(Object o){
		Res other = cast(Res)o;
		return
			this.number == other.number &&
			this.time == other.time &&
			this.id == other.id;
	}

	Result result(){
		return Result.parse(id, time);
	}
	
	char[] toText(){
		return format(">>%d %s", number, result.toMessage());
	}
	
private:
	this(int number, char[] time, char[] id){
		this.number = number;
		this.time = time;
		this.id = id;
	}
	
	int number;
	char[] time;
	char[] id;

	unittest{
		char[] text = r"
6 名前： 名前が無い程度の能力  投稿日： 2006/10/25(水) 17:05:25 [ LfG91zAw ]
    >>1
    乙かレティ

7 名前： 名前が無い程度の能力 投稿日： 2006/10/25(水) 18:09:25 [ LAD99hQs ]
    >>1
    乙かれアイス

8 名前： 名前が無い程度の能力 投稿日： 2006/10/25(水) 19:06:45 [ 6BxQK1DY ]
    >>1
    乙かレミリア

";
		Res[] results = Res.parse(text);
		assert(results.length == 3);
		assert(results[0] == new Res(6, "17:05:25", "LfG91zAw"));
		assert(results[1] == new Res(7, "18:09:25", "LAD99hQs"));
		assert(results[2] == new Res(8, "19:06:45", "6BxQK1DY"));
	}
	
	unittest{
		char[] text = r"
1 名前： 名前が無い程度の能力  投稿日： 2006/10/25(水) 16:27:01 [ uIh1PzlI ]
    2chゲサロの「勇者が魔王を倒すスレ」の東方版
    チルノが色んなキャラに必死に抗います

2 名前： 名前が無い程度の能力 投稿日： 2006/10/25(水) 16:28:45 [ uIh1PzlI ]
    前スレ
    チルノが敵を倒すスレ　第７戦
    ttp://jbbs.livedoor.jp/bbs/read.cgi/computer/6306/1159905674/

3 名前： 名前が無い程度の能力  投稿日： 2006/10/25(水) 17:05:25 [ LfG91zAw ]
    >>1
    乙かレティ

4 名前： 名前が無い程度の能力 投稿日： 2006/10/25(水) 18:09:25 [ LfG91zAw ]
    >>1
    乙かれアイス
";

		Res[] results = Res.parse(text);
		assert(results.length == 2);
	}
	
	unittest{
		assert((new Res(6, "17:05:25", "LfG91zAw")).toText() == ">>6 SpellCardBonus!! 91*5=455のダメージ。");
	}
}
